@@ -1,43 +1,32 @@
 # -*- encoding: utf-8 -*-
-import atexit
 import logging
 import os
-import sys
-from typing import AnyStr
-from typing import Dict
 from typing import List
 from typing import Optional
-import warnings
+
+import attr
 
 import ddtrace
+from ddtrace.internal import agent
+from ddtrace.internal import atexit
+from ddtrace.internal import service
 from ddtrace.internal import uwsgi
-from ddtrace.profiling import _service
+from ddtrace.internal import writer
+from ddtrace.internal.utils import formats
 from ddtrace.profiling import collector
 from ddtrace.profiling import exporter
 from ddtrace.profiling import recorder
 from ddtrace.profiling import scheduler
 from ddtrace.profiling.collector import memalloc
-from ddtrace.profiling.collector import memory
 from ddtrace.profiling.collector import stack
 from ddtrace.profiling.collector import threading
 from ddtrace.profiling.exporter import file
 from ddtrace.profiling.exporter import http
-from ddtrace.utils import deprecation
-from ddtrace.utils import formats
-from ddtrace.vendor import attr
 
 
 LOG = logging.getLogger(__name__)
 
 
-def _get_api_key():
-    legacy = os.environ.get("DD_PROFILING_API_KEY")
-    if legacy:
-        deprecation.deprecation("DD_PROFILING_API_KEY", "Use DD_API_KEY")
-        return legacy
-    return os.environ.get("DD_API_KEY")
-
-
 def _get_service_name():
     for service_name_var in ("DD_SERVICE", "DD_SERVICE_NAME", "DATADOG_SERVICE_NAME"):
         service_name = os.environ.get(service_name_var)
@@ -45,15 +34,6 @@ def _get_service_name():
             return service_name
 
 
-def gevent_patch_all(event):
-    if "ddtrace.profiling.auto" in sys.modules:
-        warnings.warn(
-            "Starting the profiler before using gevent monkey patching is not supported "
-            "and is likely to break the application. Use DD_GEVENT_PATCH_ALL=true to avoid this.",
-            RuntimeWarning,
-        )
-
-
 class Profiler(object):
     """Run profiling while code is executed.
 
@@ -74,7 +54,7 @@ class Profiler(object):
 
         if profile_children:
             try:
-                uwsgi.check_uwsgi(self.start, atexit=self.stop if stop_on_exit else None)
+                uwsgi.check_uwsgi(self._restart_on_fork, atexit=self.stop if stop_on_exit else None)
             except uwsgi.uWSGIMasterProcess:
                 # Do nothing, the start() method will be called in each worker subprocess
                 return
@@ -98,12 +78,21 @@ class Profiler(object):
 
         :param flush: Flush last profile.
         """
-        self._profiler.stop(flush)
+        atexit.unregister(self.stop)
+        try:
+            self._profiler.stop(flush)
+        except service.ServiceStatusError:
+            # Not a best practice, but for backward API compatibility that allowed to call `stop` multiple times.
+            pass
 
     def _restart_on_fork(self):
         # Be sure to stop the parent first, since it might have to e.g. unpatch functions
         # Do not flush data as we don't want to have multiple copies of the parent profile exported.
-        self.stop(flush=False)
+        try:
+            self._profiler.stop(flush=False)
+        except service.ServiceStatusError:
+            # This can happen in uWSGI mode: the children won't have the _profiler started from the master process
+            pass
         self._profiler = self._profiler.copy()
         self._profiler.start()
 
@@ -136,60 +125,8 @@ class Profiler(object):
         return self._profiler.tags
 
 
-ENDPOINT_TEMPLATE = "https://intake.profile.{}"
-
-
-def _get_default_url(
-    tracer,  # type: Optional[ddtrace.Tracer]
-    api_key,  # type: str
-):
-    # type: (...) -> str
-    """Get default profiler exporter URL.
-
-    If an API key is not specified, the URL will default to the agent location configured via the environment variables.
-
-    If an API key is specified, the profiler goes into agentless mode and uses `DD_SITE` to upload directly to Datadog
-    backend.
-
-    :param api_key: The API key provided by the user.
-    :param tracer: The tracer object to use to find default URL.
-    """
-    # Default URL changes if an API_KEY is provided in the env
-    if api_key is None:
-        if tracer is None:
-            default_hostname = "localhost"
-            default_port = 8126
-            scheme = "http"
-            path = ""
-        else:
-            default_hostname = tracer.writer._hostname
-            default_port = tracer.writer._port
-            if tracer.writer._https:
-                scheme = "https"
-                path = ""
-            elif tracer.writer._uds_path is not None:
-                scheme = "unix"
-                path = tracer.writer._uds_path
-            else:
-                scheme = "http"
-                path = ""
-
-        hostname = os.environ.get("DD_AGENT_HOST", os.environ.get("DATADOG_TRACE_AGENT_HOSTNAME", default_hostname))
-        port = int(os.environ.get("DD_TRACE_AGENT_PORT", default_port))
-
-        return os.environ.get("DD_TRACE_AGENT_URL", "%s://%s:%d%s" % (scheme, hostname, port, path))
-
-    # Agentless mode
-    legacy = os.environ.get("DD_PROFILING_API_URL")
-    if legacy:
-        deprecation.deprecation("DD_PROFILING_API_URL", "Use DD_SITE")
-        return legacy
-    site = os.environ.get("DD_SITE", "datadoghq.com")
-    return ENDPOINT_TEMPLATE.format(site)
-
-
 @attr.s
-class _ProfilerInstance(_service.Service):
+class _ProfilerInstance(service.Service):
     """A instance of the profiler.
 
     Each process must manage its own instance.
@@ -203,20 +140,16 @@ class _ProfilerInstance(_service.Service):
     env = attr.ib(factory=lambda: os.environ.get("DD_ENV"))
     version = attr.ib(factory=lambda: os.environ.get("DD_VERSION"))
     tracer = attr.ib(default=ddtrace.tracer)
+    api_key = attr.ib(factory=lambda: os.environ.get("DD_API_KEY"), type=Optional[str])
+    agentless = attr.ib(factory=lambda: formats.asbool(os.environ.get("DD_PROFILING_AGENTLESS", "False")), type=bool)
 
     _recorder = attr.ib(init=False, default=None)
     _collectors = attr.ib(init=False, default=None)
     _scheduler = attr.ib(init=False, default=None)
 
-    @staticmethod
-    def _build_default_exporters(
-        tracer,  # type: Optional[ddtrace.Tracer]
-        url,  # type: Optional[str]
-        tags,  # type: Dict[str, AnyStr]
-        service,  # type: Optional[str]
-        env,  # type: Optional[str]
-        version,  # type: Optional[str]
-    ):
+    ENDPOINT_TEMPLATE = "https://intake.profile.{}"
+
+    def _build_default_exporters(self):
         # type: (...) -> List[exporter.Exporter]
         _OUTPUT_PPROF = os.environ.get("DD_PROFILING_OUTPUT_PPROF")
         if _OUTPUT_PPROF:
@@ -224,23 +157,35 @@ class _ProfilerInstance(_service.Service):
                 file.PprofFileExporter(_OUTPUT_PPROF),
             ]
 
-        api_key = _get_api_key()
-
-        if api_key is None:
-            # Agent mode
-            endpoint_path = "/profiling/v1/input"
+        if self.url is not None:
+            endpoint = self.url
+        elif self.agentless:
+            LOG.warning(
+                "Agentless uploading is currently for internal usage only and not officially supported. "
+                "You should not enable it unless somebody at Datadog instructed you to do so."
+            )
+            endpoint = self.ENDPOINT_TEMPLATE.format(os.environ.get("DD_SITE", "datadoghq.com"))
         else:
-            endpoint_path = "/v1/input"
+            if isinstance(self.tracer.writer, writer.AgentWriter):
+                endpoint = self.tracer.writer.agent_url
+            else:
+                endpoint = agent.get_trace_url()
 
-        endpoint = _get_default_url(tracer, api_key) if url is None else url
+        if self.agentless:
+            endpoint_path = "/v1/input"
+        else:
+            # Agent mode
+            # path is relative because it is appended
+            # to the agent base path.
+            endpoint_path = "profiling/v1/input"
 
         return [
             http.PprofHTTPExporter(
-                service=service,
-                env=env,
-                tags=tags,
-                version=version,
-                api_key=api_key,
+                service=self.service,
+                env=self.env,
+                tags=self.tags,
+                version=self.version,
+                api_key=self.api_key,
                 endpoint=endpoint,
                 endpoint_path=endpoint_path,
             ),
@@ -252,9 +197,6 @@ class _ProfilerInstance(_service.Service):
                 # Allow to store up to 10 threads for 60 seconds at 100 Hz
                 stack.StackSampleEvent: 10 * 60 * 100,
                 stack.StackExceptionSampleEvent: 10 * 60 * 100,
-                # This can generate one event every 0.1s if 100% are taken — though we take 5% by default.
-                # = (60 seconds / 0.1 seconds)
-                memory.MemorySampleEvent: int(60 / 0.1),
                 # (default buffer size / interval) * export interval
                 memalloc.MemoryAllocSampleEvent: int(
                     (memalloc.MemoryCollector._DEFAULT_MAX_EVENTS / memalloc.MemoryCollector._DEFAULT_INTERVAL) * 60
@@ -265,20 +207,13 @@ class _ProfilerInstance(_service.Service):
             default_max_events=int(os.environ.get("DD_PROFILING_MAX_EVENTS", recorder.Recorder._DEFAULT_MAX_EVENTS)),
         )
 
-        if formats.asbool(os.environ.get("DD_PROFILING_MEMALLOC", "true")):
-            mem_collector = memalloc.MemoryCollector(r)
-        else:
-            mem_collector = memory.MemoryCollector(r)
-
         self._collectors = [
             stack.StackCollector(r, tracer=self.tracer),
-            mem_collector,
+            memalloc.MemoryCollector(r),
             threading.LockCollector(r, tracer=self.tracer),
         ]
 
-        exporters = self._build_default_exporters(
-            self.tracer, self.url, self.tags, self.service, self.env, self.version
-        )
+        exporters = self._build_default_exporters()
 
         if exporters:
             self._scheduler = scheduler.Scheduler(
@@ -300,9 +235,9 @@ class _ProfilerInstance(_service.Service):
             service=self.service, env=self.env, version=self.version, tracer=self.tracer, tags=self.tags
         )
 
-    def start(self):
+    def _start_service(self):  # type: ignore[override]
+        # type: (...) -> None
         """Start the profiler."""
-        super(_ProfilerInstance, self).start()
         collectors = []
         for col in self._collectors:
             try:
@@ -318,15 +253,15 @@ class _ProfilerInstance(_service.Service):
         if self._scheduler is not None:
             self._scheduler.start()
 
-    def stop(self, flush=True):
+    def _stop_service(  # type: ignore[override]
+        self, flush=True  # type: bool
+    ):
+        # type: (...) -> None
         """Stop the profiler.
 
         :param flush: Flush a last profile.
         """
-        if self.status != _service.ServiceStatus.RUNNING:
-            return
-
-        if self._scheduler:
+        if self._scheduler is not None:
             self._scheduler.stop()
             # Wait for the export to be over: export might need collectors (e.g., for snapshot) so we can't stop
             # collectors before the possibly running flush is finished.
@@ -336,14 +271,11 @@ class _ProfilerInstance(_service.Service):
                 self._scheduler.flush()
 
         for col in reversed(self._collectors):
-            col.stop()
+            try:
+                col.stop()
+            except service.ServiceStatusError:
+                # It's possible some collector failed to start, ignore failure to stop
+                pass
 
         for col in reversed(self._collectors):
             col.join()
-
-        # Python 2 does not have unregister
-        if hasattr(atexit, "unregister"):
-            # You can unregister a method that was not registered, so no need to do any other check
-            atexit.unregister(self.stop)
-
-        super(_ProfilerInstance, self).stop()
